import datetime as dt
from sunpeek.common.unit_uncertainty import Q
from sunpeek.components.components_factories import Collector, CollectorSST, CollectorQDT, CollectorTypes
import sunpeek.components.iam_methods as iam

all_definitions = [
    CollectorQDT(name="Arcon 3510",
                 manufacturer_name="Arcon-Sunmark A/S",
                 product_name="HTHEATstore 35/10",
                 licence_number='SP SC0843-14',
                 test_report_id="6P02267-C-Rev 1 (2016-07-06), 4P04266-C-Rev 2 (2015-11-10)",
                 certificate_date_issued=dt.datetime(2016, 7, 14),
                 certificate_lab='SP Technical Research Institute of Sweden',
                 collector_type=CollectorTypes.flat_plate.value,
                 description="Cover: single-glazed & foil. Absorber: harp. Hydraulics: Non-Tichelmann",
                 test_reference_area="gross",
                 area_gr=Q(13.57, "m**2"),
                 gross_width=Q(5973, "mm"),
                 gross_length=Q(2272, "mm"),
                 gross_height=Q(145, "mm"),
                 a1=Q(2.067, "W m**-2 K**-1"),
                 a2=Q(0.009, "W m**-2 K**-2"),
                 a5=Q(7.313, "kJ m**-2 K**-1"),
                 kd=Q(0.93, ""),
                 eta0b=Q(0.745, ""),
                 f_prime=Q(0.95, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.94, 0.9, 0.82, 0.65, 0.32, 0]))
                 ),
    CollectorQDT(name='powerSol 55',
                 manufacturer_name="Gasokol GmbH",
                 product_name="powerSol 55",
                 licence_number='011-7S2928 F',
                 test_report_id="21246087.001 (2019-05-24)",
                 certificate_date_issued=dt.datetime(2019, 6, 4),
                 certificate_lab="TÜV Rheinland Energy GmbH, Germany",
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area='gross',
                 area_gr=Q(5.46, 'm**2'),
                 area_ap=Q(5.04, 'm**2'),
                 gross_width=Q(2522, "mm"),
                 gross_length=Q(2166, "mm"),
                 gross_height=Q(150, "mm"),
                 a1=Q(2.34, "W m**-2 K**-1"),
                 a2=Q(0.012, "W m**-2 K**-2"),
                 a5=Q(13.489, "kJ m**-2 K**-1"),
                 kd=Q(0.88, ""),
                 eta0b=Q(0.741, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.94, 0.89, 0.8, 0.61, 0.3, 0]))
                 ),
    CollectorQDT(name='powerSol 120',
                 manufacturer_name="Gasokol GmbH",
                 product_name="powerSol 120",
                 licence_number='011-7S2928 F',
                 test_report_id="21246087.001 (2019-05-24)",
                 certificate_date_issued=dt.datetime(2019, 6, 4),
                 certificate_lab="TÜV Rheinland Energy GmbH, Germany",
                 collector_type=CollectorTypes.flat_plate.value,
                 area_gr=Q(12.0, 'm**2'),
                 area_ap=Q(11.0, 'm**2'),
                 test_reference_area='gross',
                 gross_width=Q(5770, "mm"),
                 gross_length=Q(2080, "mm"),
                 gross_height=Q(150, "mm"),
                 a1=Q(2.34, "W m**-2 K**-1"),
                 a2=Q(0.012, "W m**-2 K**-2"),
                 a5=Q(13.489, "kJ m**-2 K**-1"),
                 kd=Q(0.88, ""),
                 eta0b=Q(0.741, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.94, 0.89, 0.8, 0.61, 0.3, 0]))
                 ),
    CollectorQDT(name='powerSol 136',
                 manufacturer_name="Gasokol GmbH",
                 product_name="powerSol 136",
                 licence_number='011-7S2928 F',
                 test_report_id="21246087.001 (2019-05-24)",
                 certificate_date_issued=dt.datetime(2019, 6, 4),
                 test_reference_area='gross',
                 certificate_lab="TÜV Rheinland Energy GmbH, Germany",
                 collector_type=CollectorTypes.flat_plate.value,
                 area_gr=Q(13.59, 'm**2'),
                 area_ap=Q(12.50, 'm**2'),
                 gross_width=Q(6275, "mm"),
                 gross_length=Q(2166, "mm"),
                 gross_height=Q(150, "mm"),
                 a1=Q(2.34, "W m**-2 K**-1"),
                 a2=Q(0.012, "W m**-2 K**-2"),
                 a5=Q(13.489, "kJ m**-2 K**-1"),
                 kd=Q(0.88, ""),
                 eta0b=Q(0.741, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.94, 0.89, 0.8, 0.61, 0.3, 0]))
                 ),
    CollectorQDT(name='Greenonetec 3803',
                 manufacturer_name="GREENoneTEC Solarindustrie GmbH",
                 product_name='GK 3803',
                 licence_number='011-7S2565 F',
                 test_report_id='15COL1247 (2015-08-28), 15COL1247Q (2015-08-28)',
                 certificate_date_issued=dt.datetime(2015, 8, 28),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="aperture",
                 area_gr=Q(7.91, 'm**2'),
                 area_ap=Q(7.41, 'm**2'),
                 gross_width=Q(3557, "mm"),
                 gross_length=Q(2224, "mm"),
                 gross_height=Q(135, "mm"),
                 a1=Q(2.102, "W m**-2 K**-1"),
                 a2=Q(0.016, "W m**-2 K**-2"),
                 a5=Q(9.664, "kJ m**-2 K**-1"),
                 kd=Q(0.931, ""),
                 eta0b=Q(0.814, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.98, 0.96, 0.91, 0.82, 0.53, 0.27, 0]))
                 ),
    CollectorQDT(name='Greenonetec 3803S',
                 manufacturer_name="GREENoneTEC Solarindustrie GmbH",
                 product_name='GK 3803S',
                 licence_number='011-7S2566 F',
                 test_report_id='15COL1257 (2015-08-28), 15COL1257Q (2015-08-28)',
                 certificate_date_issued=dt.datetime(2015, 8, 28),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="aperture",
                 area_ap=Q(7.41, 'm**2'),
                 area_gr=Q(7.91, 'm**2'),
                 gross_width=Q(3557, "mm"),
                 gross_length=Q(2224, "mm"),
                 gross_height=Q(135, "mm"),
                 a1=Q(3.083, "W m**-2 K**-1"),
                 a2=Q(0.013, "W m**-2 K**-2"),
                 a5=Q(9.985, "kJ m**-2 K**-1"),
                 kd=Q(0.918, ""),
                 eta0b=Q(0.857, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.95, 0.91, 0.83, 0.68, 0.21, 0]))
                 ),
    CollectorQDT(name='Greenonetec 3133',
                 manufacturer_name="GREENoneTEC Solarindustrie GmbH",
                 product_name='GK 3133',
                 licence_number='011-7S2565 F',
                 test_report_id='15COL1247 (2015-08-28), 15COL1247Q (2015-08-28)',
                 certificate_date_issued=dt.datetime(2015, 8, 28),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="aperture",
                 area_gr=Q(13.17, 'm**2'),
                 area_ap=Q(12.35, 'm**2'),
                 gross_width=Q(5920, "mm"),
                 gross_length=Q(2224, "mm"),
                 gross_height=Q(135, "mm"),
                 a1=Q(2.102, "W m**-2 K**-1"),
                 a2=Q(0.016, "W m**-2 K**-2"),
                 a5=Q(9.664, "kJ m**-2 K**-1"),
                 kd=Q(0.931, ""),
                 eta0b=Q(0.814, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.98, 0.96, 0.91, 0.82, 0.53, 0.27, 0]))
                 ),
    CollectorQDT(name='Greenonetec 3133S',
                 manufacturer_name="GREENoneTEC Solarindustrie GmbH",
                 product_name='GK 3133S',
                 licence_number='011-7S2566 F',
                 test_report_id='15COL1257 (2015-08-28), 15COL1257Q (2015-08-28)',
                 certificate_date_issued=dt.datetime(2015, 8, 28),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="aperture",
                 area_gr=Q(13.17, 'm**2'),
                 area_ap=Q(12.35, 'm**2'),
                 gross_width=Q(5920, "mm"),
                 gross_length=Q(2224, "mm"),
                 gross_height=Q(135, "mm"),
                 a1=Q(3.083, "W m**-2 K**-1"),
                 a2=Q(0.013, "W m**-2 K**-2"),
                 a5=Q(9.985, "kJ m**-2 K**-1"),
                 kd=Q(0.918, ""),
                 eta0b=Q(0.857, ""),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.95, 0.91, 0.83, 0.68, 0.21, 0]))
                 ),
    CollectorQDT(name='KBB K5-3Giga+',
                 manufacturer_name="KBB Kollektorbau GmbH",
                 product_name='KG-3Giga+',
                 licence_number='011-7S2437 F',
                 test_report_id='16COL1338 (2016-11-29), 16COL1338Q (2016-11-29)',
                 certificate_date_issued=dt.datetime(2016, 12, 2),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area='gross',
                 area_gr=Q(7.45, 'm**2'),
                 gross_width=Q(3450, "mm"),
                 gross_length=Q(2160, "mm"),
                 gross_height=Q(119, "mm"),
                 eta0b=Q(0.752, ''),
                 a1=Q(2.416, 'W m**-2 K**-1'),
                 a2=Q(0.008, 'W m**-2 K**-2'),
                 a5=Q(8.3, "kJ m**-2 K**-1"),
                 kd=Q(0.964, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 1, 0.99, 0.98, 0.94, 0.83, 0.56, 0.28, 0]))
                 ),
    CollectorQDT(name='KBB K5-4Giga+',
                 manufacturer_name="KBB Kollektorbau GmbH",
                 product_name='KG-4Giga+',
                 licence_number='011-7S2437 F',
                 test_report_id='16COL1338 (2016-11-29), 16COL1338Q (2016-11-29)',
                 certificate_date_issued=dt.datetime(2016, 12, 2),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(9.94, 'm**2'),
                 gross_width=Q(4600, "mm"),
                 gross_length=Q(2160, "mm"),
                 gross_height=Q(119, "mm"),
                 eta0b=Q(0.752, ''),
                 a1=Q(2.416, 'W m**-2 K**-1'),
                 a2=Q(0.008, 'W m**-2 K**-2'),
                 a5=Q(8.3, "kJ m**-2 K**-1"),
                 kd=Q(0.964, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 1, 0.99, 0.98, 0.94, 0.83, 0.56, 0.28, 0]))
                 ),
    CollectorQDT(name='KBB K5Giga+',
                 manufacturer_name="KBB Kollektorbau GmbH",
                 product_name='K5Giga+',
                 licence_number='011-7S2437 F',
                 test_report_id='16COL1338 (2016-11-29), 16COL1338Q (2016-11-29)',
                 certificate_date_issued=dt.datetime(2016, 12, 2),
                 certificate_lab='TZS, ITW University Stuttgart, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(12.42, 'm**2'),
                 gross_width=Q(5750, "mm"),
                 gross_length=Q(2160, "mm"),
                 gross_height=Q(119, "mm"),
                 eta0b=Q(0.752, ''),
                 a1=Q(2.416, 'W m**-2 K**-1'),
                 a2=Q(0.008, 'W m**-2 K**-2'),
                 a5=Q(8.3, "kJ m**-2 K**-1"),
                 kd=Q(0.964, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 1, 0.99, 0.98, 0.94, 0.83, 0.56, 0.28, 0]))
                 ),
    CollectorQDT(name='ökoTech HT 16.7 Export',
                 manufacturer_name="ökoTech Solarkollektoren GmbH",
                 product_name='HT',
                 licence_number='011-7S837 F',
                 test_report_id='2.04.01120.1.0- QT, 2.04.01120.1.0- LT, 2.04.00667.1.0-2 - QT',
                 certificate_date_issued=dt.datetime(2017, 2, 7),
                 certificate_lab='TÜV Rheinland Energy GmbH, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(16.7, 'm**2'),
                 gross_width=Q(7170, "mm"),
                 gross_length=Q(2330, "mm"),
                 gross_height=Q(187, "mm"),
                 eta0b=Q(0.701, ''),
                 a1=Q(2.582, 'W m**-2 K**-1'),
                 a2=Q(0.005, 'W m**-2 K**-2'),
                 a5=Q(14.7, "kJ m**-2 K**-1"),
                 kd=Q(0.983, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.93, 0.88, 0.78, 0.58, 0.0]))
                 ),
    CollectorQDT(name='ökoTech HT 4.2 Export',
                 manufacturer_name="ökoTech Solarkollektoren GmbH",
                 product_name='HT',
                 licence_number='011-7S837 F',
                 test_report_id='2.04.01120.1.0- QT, 2.04.01120.1.0- LT, 2.04.00667.1.0-2 - QT',
                 certificate_date_issued=dt.datetime(2017, 2, 7),
                 certificate_lab='TÜV Rheinland Energy GmbH, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(4.2, 'm**2'),
                 gross_width=Q(2076, "mm"),
                 gross_length=Q(2050, "mm"),
                 gross_height=Q(187, "mm"),
                 eta0b=Q(0.701, ''),
                 a1=Q(2.582, 'W m**-2 K**-1'),
                 a2=Q(0.005, 'W m**-2 K**-2'),
                 a5=Q(14.7, "kJ m**-2 K**-1"),
                 kd=Q(0.983, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.97, 0.93, 0.88, 0.78, 0.58, 0.0]))
                 ),
    CollectorQDT(name='ensol DIS 150',
                 manufacturer_name="Energetyka Solarna ensol Sp. z.o.o.",
                 product_name='DIS 150',
                 licence_number='011-7S2978 F',
                 test_report_id='21249150.001',
                 certificate_date_issued=dt.datetime(2020, 7, 20),
                 certificate_lab='TÜV Rheinland Energy GmbH, Germany',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(15.5, 'm**2'),
                 gross_width=Q(6606, "mm"),
                 gross_length=Q(2350, "mm"),
                 gross_height=Q(173, "mm"),
                 eta0b=Q(0.765, ''),
                 a1=Q(2.23, 'W m**-2 K**-1'),
                 a2=Q(0.008, 'W m**-2 K**-2'),
                 a5=Q(6.483, "kJ m**-2 K**-1"),
                 kd=Q(0.91, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 0.99, 0.98, 0.95, 0.91, 0.84, 0.70, 0.35, 0]))
                 ),
    CollectorQDT(name='Sunrain FPC1500C',
                 manufacturer_name="Jiangsu Sunrain Solar Energy Co., Ltd",
                 product_name='FPC1500C',
                 licence_number='011-7S3004 F',
                 test_report_id='201203085GZU-001',
                 certificate_date_issued=dt.datetime(2020, 2, 25),
                 certificate_lab='DIN CERTCO',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(15.02, 'm**2'),
                 gross_width=Q(5960, "mm"),
                 gross_length=Q(2520, "mm"),
                 gross_height=Q(166, "mm"),
                 eta0b=Q(0.852, ''),
                 a1=Q(3.03, 'W m**-2 K**-1'),
                 a2=Q(0.014, 'W m**-2 K**-2'),
                 a5=Q(6.833, "kJ m**-2 K**-1"),
                 kd=Q(0.92, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 1, 1, 0.99, 0.96, 0.9, 0.78, 0.52, 0.0]))
                 ),
    CollectorQDT(name='TVP MT-Power v4',
                 manufacturer_name='TVP Solar SA',
                 product_name='MT- Power v4',
                 licence_number='011-7S1890F',
                 test_report_id='16COL1343, 16COL1343Q',
                 certificate_date_issued=dt.datetime(2017, 5, 24),
                 certificate_lab='DIN CERTCO',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area='gross',
                 area_gr=Q(1.96, 'm**2'),
                 gross_length=Q(975, 'mm'),
                 gross_width=Q(2015, 'mm'),
                 gross_height=Q(51, 'mm'),
                 eta0b=Q(0.737, ''),
                 a1=Q(0.504, 'W m**-2 K**-1'),
                 a2=Q(0.006, 'W m**-2 K**-2'),
                 a5=Q(15.32, "kJ m**-2 K**-1"),
                 kd=Q(0.957, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([1, 1, 0.99, 0.98, 0.95, 0.88, 0.72, 0.36, 0.0]))
                 ),
    CollectorQDT(name='Absolicon T160',
                 manufacturer_name='Absolicon Solar Collector AB',
                 product_name='T160',
                 licence_number='011-7S2902 C',
                 test_report_id='C1730ISO',
                 certificate_date_issued=dt.datetime(2019, 1, 15),
                 certificate_lab='DIN CERTCO',
                 collector_type=CollectorTypes.concentrating.value,
                 test_reference_area='gross',
                 area_gr=Q(6.04, 'm**2'),
                 gross_length=Q(1095, 'mm'),
                 gross_width=Q(5514, 'mm'),
                 gross_height=Q(5.51, 'mm'),
                 eta0b=Q(0.697, ''),
                 a1=Q(0.73, 'W m**-2 K**-1'),
                 a2=Q(0.000, 'W m**-2 K**-2'),
                 a5=Q(1483, "J m**-2 K**-1"),
                 kd=Q(0.12, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([[1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 0],
                                                                  [0.99, 0.99, 0.98, 0.96, 0.91, 0.77, 0.53, 0.18, 0]]))
                 ),

    CollectorQDT(name='Ako Tec MEGA 26',
                 manufacturer_name='Ako Tec Produktionsgesellschaft',
                 product_name='MEGA 26',
                 licence_number='011-7S2827 R',
                 test_report_id='21242732.001rev1',
                 certificate_date_issued=dt.datetime(2019, 2, 1),
                 certificate_lab='TÜV Rheinland Energy GmbH',
                 collector_type=CollectorTypes.concentrating.value,
                 test_reference_area='gross',
                 area_gr=Q(4.33, 'm**2'),
                 gross_length=Q(1983, 'mm'),
                 gross_width=Q(2184, 'mm'),
                 eta0b=Q(0.483, ''),
                 a1=Q(0.63, 'W m**-2 K**-1'),
                 a2=Q(0.000, 'W m**-2 K**-2'),
                 a5=Q(8136, "J m**-2 K**-1"),
                 kd=Q(1.1, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([[1.24, 1.25, 1.29, 1.08, 1.23, 1.18, 1.15, 0.97, 0],
                                                                  [1.00, 1.00, 0.99, 0.98, 0.97, 0.94, 0.89, 0.44, 0]]))
                 ),

    CollectorQDT(name='Ako Tec MEGA 78',
                 manufacturer_name='Ako Tec Produktionsgesellschaft',
                 product_name='MEGA 78',
                 licence_number='011-7S2827 R',
                 test_report_id='21242732.001rev1',
                 certificate_date_issued=dt.datetime(2019, 2, 1),
                 certificate_lab='TÜV Rheinland Energy GmbH',
                 collector_type=CollectorTypes.concentrating.value,
                 test_reference_area='gross',
                 area_gr=Q(12.99, 'm**2'),
                 gross_length=Q(5950, 'mm'),
                 gross_width=Q(2184, 'mm'),
                 eta0b=Q(0.483, ''),
                 a1=Q(0.63, 'W m**-2 K**-1'),
                 a2=Q(0.000, 'W m**-2 K**-2'),
                 a5=Q(8136, "J m**-2 K**-1"),
                 kd=Q(1.1, ''),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([[1.24, 1.25, 1.29, 1.08, 1.23, 1.18, 1.15, 0.97, 0],
                                                                  [1.00, 1.00, 0.99, 0.98, 0.97, 0.94, 0.89, 0.44, 0]]))
                 ),

    # Collectors tested with static test ----------------------------------------------------------------------------
    # Overwritten by Collector below, because CollectorSST does not allow for an eta0b parameter.
    # CollectorSST(name='Greenonetec HT 13,6',
    #                  manufacturer_name="GREENoneTEC Solarindustrie GmbH",
    #                  product_name='GK HT 13,6',
    #                  licence_number='011-7S2819 F',
    #                  test_report_id='20COLC1735ISO (2020-11-10), 20COLC1735ISOQ (2010-11-10), C1735L, '
    #                                 'C1781L (2018-09-06)',
    #                  certificate_date_issued=dt.datetime(2020, 11, 10),
    #                  certificate_lab='TZS, ITW University Stuttgart, Germany',
    #                  test_reference_area="gross",
    #                  area_gr=Q(13.61, 'm**2'),
    #                  gross_width=Q(5970, "mm"),
    #                  gross_length=Q(2280, "mm"),
    #                  gross_height=Q(185, "mm"),
    #                  eta0hem=Q(0.782),  # Misleading solar keymark data sheet: eta0hem or eta0b?
    #                  a1=Q(2.27, "W m**-2 K**-1"),
    #                  a2=Q(0.018, "W m**-2 K**-2"),
    #                  ceff=Q(5980, "J m**-2 K**-1"),
    #                  kd=Q(0.92, ""),
    #                  iam_method=iam.IAM_K50(Q(0.97))
    #                  ),
    Collector(name='Greenonetec HT 13,6',
              manufacturer_name="GREENoneTEC Solarindustrie GmbH",
              product_name='GK HT 13,6',
              licence_number='011-7S2819 F',
              test_report_id='20COLC1735ISO (2020-11-10), 20COLC1735ISOQ (2010-11-10), C1735L, '
                             'C1781L (2018-09-06)',
              certificate_date_issued=dt.datetime(2020, 11, 10),
              certificate_lab='TZS, ITW University Stuttgart, Germany',
              collector_type=CollectorTypes.flat_plate.value,
              test_type='SST',
              test_reference_area='gross',
              area_gr=Q(13.61, 'm**2'),
              gross_width=Q(5970, "mm"),
              gross_length=Q(2280, "mm"),
              gross_height=Q(185, "mm"),
              eta0b=Q(0.782),
              a1=Q(2.27, "W m**-2 K**-1"),
              a2=Q(0.018, "W m**-2 K**-2"),
              a5=Q(5980, "J m**-2 K**-1"),
              kd=Q(0.92, ""),
              iam_method=iam.IAM_K50(Q(0.97))
              ),
    CollectorSST(name="Savo SF500-15",
                 manufacturer_name="Savo Solar Oyj",
                 product_name="Savo SF500-15",
                 licence_number='011-7S2688 F',
                 test_report_id="C1704LPEN (2016-09-28), C1704QPEN (2016-09-28)",
                 certificate_date_issued=dt.datetime(2016, 10, 7),
                 certificate_lab='SPF, Switzerland',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(15.96, 'm**2'),
                 gross_width=Q(6158, "mm"),
                 gross_length=Q(2591, "mm"),
                 gross_height=Q(213, "mm"),
                 eta0hem=Q(0.812, ''),
                 a1=Q(2.936, 'W m**-2 K**-1'),
                 a2=Q(0.009, 'W m**-2 K**-2'),
                 ceff=Q(10.2, "kJ m**-2 K**-1"),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([[1, 1, 1, 0.99, 0.97, 0.91, 0.75, 0.42, 0],
                                                                  [1, 1, 1, 0.99, 0.98, 0.94, 0.84, 0.59, 0]]))
                 ),
    CollectorSST(name="Savo SF500-15DG",
                 manufacturer_name="Savo Solar Oyj",
                 product_name="Savo SF500-15DG",
                 licence_number='011-7S2689 F',
                 test_report_id="C1705LPEN (2016-12-15), C1705QPEN (2016-12-15)",
                 certificate_date_issued=dt.datetime(2016, 12, 15),
                 certificate_lab='SPF, Switzerland',
                 collector_type=CollectorTypes.flat_plate.value,
                 test_reference_area="gross",
                 area_gr=Q(15.96, 'm**2'),
                 gross_width=Q(6158, "mm"),
                 gross_length=Q(2591, "mm"),
                 gross_height=Q(213, "mm"),
                 eta0hem=Q(0.793, ''),
                 a1=Q(2.520, 'W m**-2 K**-1'),
                 a2=Q(0.004, 'W m**-2 K**-2'),
                 ceff=Q(12.0, "kJ m**-2 K**-1"),
                 iam_method=iam.IAM_Interpolated(aoi_reference=Q([10, 20, 30, 40, 50, 60, 70, 80, 90], 'deg'),
                                                 iam_reference=Q([[1, 1, 0.99, 0.98, 0.96, 0.87, 0.68, 0.38, 0],
                                                                  [1, 1, 1.00, 0.99, 0.96, 0.91, 0.78, 0.53, 0]]))
                 ),
    CollectorSST(name="CONA CCS+",
                 manufacturer_name="CONA Entwicklungs- u. Handelsges.m.b.H.",
                 product_name="CCS+",
                 licence_number='011-7S2471 L',
                 test_report_id="ktb-2014-27",
                 certificate_date_issued=dt.datetime(2015, 1, 14),
                 certificate_lab='Fraunhofer ISE',
                 test_reference_area="aperture",   # <-- aperture
                 area_ap=Q(1.92, 'm**2'),
                 area_gr=Q(2.05, 'm**2'),
                 gross_length=Q(1012, "mm"),
                 gross_width=Q(2024, "mm"),
                 gross_height=Q(90, "mm"),
                 eta0hem=Q(0.772, ''),  # <-- eta0hem, no eta0b
                 a1=Q(11.247, 'W m**-2 K**-1'),
                 a2=Q(0, 'W m**-2 K**-2'),
                 a8=None,
                 ceff=Q(16.2, "kJ m**-2 K**-1"),
                 iam_method=iam.IAM_K50(k50=Q(0.89)),
                 collector_type=CollectorTypes.flat_plate.value,
                 ),
]


def get_definition(name):
    for coll_type in all_definitions:
        if coll_type.name == name:
            return coll_type
